
*	DSP1.ASM
*
*	TMS320C25 SOURCE CODE FOR XASM25 ASSEMBLER
*
*	CONVERTS AND DECODES WEFAX APT AUDIO SIGNAL
*

*
* ----	REVISIONS ----------------------------------------------
*
* V1.0	09/SEP/94 First full version with:
*		  * Start tone detection with hardware based AGC
*		  * Header detection with software fine AGC
*		  * Header phase tracking & image detection
*		  * Image phase tracking
* V1.1  15/11/94  Shift image a bit by re-phasing each line

*
* ----	EQUATES  -----------------------------------------------
*

*
*	CONSTANTS
*

DEFG	EQU	255		; DEFAULT AGC GAIN SETTING

*
*	PAGE 0 DMAs
*

DRR	EQU	0		; SERIAL DATA RX REG
DXR	EQU	1		; SERIAL DATA TX REG
TIM	EQU	2		; COUNTER REG
PRD	EQU	3		; COUNTER RE-LOAD / PERIOD REG
IMR	EQU	4		; INTERRUPT MASK REG
GREG	EQU	5		; GLOBAL MEMORY ALLOC REG

*
*	BLOCK 2 RAM (32 BYTES)
*	STARTS AT >60 .. >7F         = PAGE 0
*
BLOCK2	EQU	>60

SSTSAV	EQU	BLOCK2+0	; SAVES SST AT INTERRUPT
TEMP	EQU	BLOCK2+1	; TEMP STORAGE
GAIN	EQU	BLOCK2+2	; AGC GAIN SETTING
WHTREF	EQU	BLOCK2+3	; 9 BIT WHITE LEVEL REFERENCE
BLKREF	EQU	BLOCK2+4	; BLACK LEVEL REFERENCE
PCOUNT	EQU	BLOCK2+5	; PIXEL COUNTER
ADJ	EQU	BLOCK2+6	; PHASE ADJUST ACCUMULATOR

BUF16	EQU	BLOCK2+16	; 16 BYTE GENERAL TEMP BUFFER

*
*	BLOCKS 0 & 1 RAM ( 2 X 256 BYTES)
*	START AT >200 ... >3FF       = PAGES 4,5,6,7
*
BLOCK0	EQU	>200
BLOCK1	EQU	>300

*
* ----	VECTORS  -----------------------------------------------
*

	AORG	>0000

RESET	B	INIT
	B	INT0
	B	INT1	
	B	INT2

	AORG	24

	B	TINT
	B	RINT
	B	XINT
	B	USER

*
* ----	NULL INTERRUPT HANDLERS  -------------------------------
*

INT1	EINT
	RET

INT2	EINT
	RET

TINT	EINT
	RET

XINT	EINT
	RET

USER	EINT
	RET

RINT	EINT
	RET

*
*	INT0 INTERRUPT HANDLER
*

INT0	SST	SSTSAV
	LARP	AR7
	IN	*+,PA0
	LDPK	0
	LST	SSTSAV
	EINT
	RET

*
* ----	INITIALISATION  ----------------------------------------
*

INIT	DINT			; DISSABLE INTERRUPTS
	SXF			; CLOSE PING PONG BUFFER
	SOVM			; OVERFLOW SATURATION MODE
	CNFD			; RAM BLOCK 0 = DATA MEMORY
	LDPK	0		; DATA PAGE 0
	LACK	?000001		; INT0 ENABLED
	SACL	IMR		; SETUP INTs 
	ZAC			; 0 -> ACCL
	SACL	GREG		; NO GLOBAL MEMORY
	FORT	1		; 8 BIT SERIAL DATA
	SFSM			; SERIAL RX REQUIRES FRAMING
	STXM			; SET TX FSR TO OUTPUT MODE
	SHM			; NORMAL HALT MODE
	SSXM			; SIGN EXTEND MODE
	SPM	0		; NO MPY SHIFTS

*
*	RAM INITIALISATION
*

	ZAC			; CLEAR TO ZERO
	LARP	AR7		; REG POINTER TO AR7
	LARK	AR7,BLOCK2	; CLEAR BLOCK 2
	RPTK	31
	SACL	*+
	LRLK	AR7,BLOCK0	; CLEAR BLOCK 0
	RPTK	255
	SACL	*+
	RPTK	255		; & BLOCK 1
	SACL	*+

*
*	IO INITIALISATION
*

	LDPK	0		; MEM BLOCK 2
	LARK	AR0,DEFG	; 255 => AGC, GAIN := 1
	SAR	AR0,GAIN	; SET IT
	OUT	GAIN,PA0	; PORT0 = GAIN O/P PORT

*
*	INTERRUPT SERVICE INITIALISATION
*

	LRLK	AR7,BLOCK0	; AR7 RESERVED FOR INT0 SERVICE
	EINT

*
* ----	MAIN  --------------------------------------------------
*

*
*	The start tone detector samples at pix_rate/2.8
*	to give 4f @ start tone of 300Hz (11.2pix/cycle).
*	So... the 16 entry buffer will contain 4 peaks.
*

AGCLP0	LARK	AR0,0
	SAR	AR0,TEMP	; USE TEMP TO HOLD PEAK
	LARK	AR2,BUF16
	CALL	SAMPLE		; ADD A SLIGHT PHASE SHIFT..
	CALL	SHFT20		; ..BEFORE EACH NEW PASS
FPKLP	CALL	SAMPLE		; TAKE SLOW SAMPLES
	CALL	SHFT56		; EVERY 2.8 PIXELS
	LARP	AR2		; USE POINTER
	SACL	*		; SAVE SAMPLE TO BUFFER
	LAR	AR1,*+		; GET NEXT SAMPLE
	LAR	AR0,TEMP	; GET CURRENT MAX SAMPLE
	LARP	AR1		; USE SAMPLE
	CMPR	2		; SET TC IF (ARP) > AR0
	BBZ	FPKSKP		; Z IF NOT GREATER
	SAR	AR1,TEMP	; UPDATE MAX
FPKSKP	LARP	AR2		; DONE ?
	LARK	AR0,BUF16+16	; EOB
	CMPR	0		; = ?
	BBZ	FPKLP		; LOOP TILL ALL CHECKED

*	OK, TEMP := MAX, Now XC for 300Hz

	MPYK	0		; P := 0
	ZAC			; ACCL := 0
	LARK	AR2,BUF16
	LARP	AR2
	RPTK	15		; 16 MACs
	MAC	XC300,*+
	APAC			; ADD IN LAST RESULT
	ABS			; TAKE THE ABS VALUE
	SACL	BUF16		; XC -> BUF16's OLDEST POINT

*	Before comparing XC and MAX check for clipping...

	LAR	AR1,TEMP	; GET MAX
	LARP	AR1		; USE AR1
	LRLK	AR0,510		; >= 511 ?
	CMPR	2		; SET TC IF MAX >= 511
	BBZ	AGCSK1		; ELSE SKIP PANIC MEASURES

*	CLIPPING ALERT....

	LAC	GAIN		; HIGHER THE VALUE, LOWER THE GAIN
	ADDK	255		; AVERAGE IT WITH 255
	SFR
	SACL	GAIN		; UPDATE MEMORY
	OUT	GAIN,PA0	; SET NEW GAIN
	B	AGCLP0		; KEEP GOIN'

*	Now we can safely look for 300Hz
*	If XC > 3xPk then bingo.

AGCSK1	LAC	TEMP		; MAX -> ACCL
	ADD	TEMP,1		; ACC := 3 x MAX
	SACL	TEMP		; REPLACE OLD VERSION
	LAR	AR1,TEMP	; MAX x 3
	LAR	AR0,BUF16	; XC
	LARP	AR1
	CMPR	2		; SET TC IF MAX3 > XC
	BBNZ	AGCLP0

*
*	HEADER TONE DETECTED

*
*	AGC ALGORITHM:
*
*	GAIN := (CURRENT_GAIN*ACTUAL_MAX/DESIRED_MAX) / 8
*	      +  CURRENT_GAIN        *        7       / 8
*	& DESIRED_MAX = 1024
*

	LAC	TEMP		; ACCL = MAX3
	ADLK	7168		; ADD 7 ( * 1024 )
	SACL	TEMP
	LT	TEMP
	MPY	GAIN
	ZAC
	APAC
	SFL
	SFL
	SFL
	LAR	AR1,GAIN
	SACH	GAIN
	OUT	GAIN,PA0

*	HAS GAIN STABALISED YET ?

	LAR	AR0,GAIN
	LARP	AR1
	CMPR	0
	BBZ	AGCLP0

*
*	COURSE GAIN SET, NOW HUNT HEADER
*	AND MEASURE LONG TERM AVS -> WHTREF & BLKREF

*	FIRST, LOOK FOR A RUN OF 16 WHITE PIXELS...

HDLP0	LARK	AR1,0		; RUN LENGTH COUNTER
HDLP1	CALL	SAMPLE
	CALL	SHFT20
	SUBK	200		; THRESHOLD
	BLZ	HDLP0		; SAMPLE < 200, RE-START
	LARP	AR1
	MAR	*+
	LARK	AR0,16		; GOT TO 16 YET ?
	CMPR	0
	BBZ	HDLP1		; KEEP LOOPING

*	NOW ACCUMULATE THE NEXT 64 -> WHTREF
*	CHECKING THAT THEY ARE ALL WHITE.

	ZAC
	SACL	WHTREF
	LARK	AR1,0		; SAMPLE COUNTER
WHTLP0	CALL	SAMPLE
	CALL	SHFT20
	SUBK	200
	BLZ	HDLP0		; A DARK PIXEL, RE-START
	ADDK	200		; REPAIR ACCL
	ADD	WHTREF
	SACL	WHTREF		; ADD INTO WHTREF
	LARP	AR1
	MAR	*+
	LARK	AR0,64		; ALL DONE ?
	CMPR	0
	BBZ	WHTLP0		; KEEP LOOPING

*	DIV ACCL BY 64 -> WHTREF

	RPTK	5		; 6 TIMES
	SFR			; DIV 2^6
	SACL	WHTREF		; SAVE RESULT

*	WAIT FOR A BLACK PIXEL

BLKLP0	CALL	SAMPLE
	CALL	SHFT20
	SUBK	100		; THRESHOLD
	BGZ	BLKLP0		; SAMPLE > 200, KEEP LOOKING

*	ALIGN PIXCOUNT TO IT

	LACK	2
	SACL	PCOUNT		; INIT PCOUNT

*	NOW ACCUMULATE THE NEXT 32 -> BLKREF
*	CHECKING THAT THEY ARE ALL BLACK.

	ZAC
	SACL	BLKREF
	LARK	AR1,0		; SAMPLE COUNTER
BLKLP1	CALL	SAMPLE
	CALL	SHFT20
	SUBK	100
	BGZ	BLKLP0		; A WHITE PIXEL, RE-START
	ADDK	100		; REPAIR ACCL
	ADD	BLKREF
	SACL	BLKREF		; ADD INTO BLKREF
	LARP	AR1
	MAR	*+
	LARK	AR0,32		; ALL DONE ?
	CMPR	0
	BBZ	BLKLP1		; KEEP LOOPING

*	DIV ACCL BY 32 -> BLKREF

	RPTK	4		; 5 TIMES
	SFR			; DIV 2^5
	SACL	BLKREF		; SAVE RESULT

*	RE-DEFINE WHTREF TO BE THE SCALE FACTOR

	LAC	WHTREF
	SUB	BLKREF
	SFR			; DIV BY 2, SEE SCALE
	SACL	WHTREF		; WHT := (WHT-BLK)/2

*	DUMP SAMPLES TIL PIX 832 COMES UP...

HDLP2	CALL	SAMPLE
	CALL	SHFT20
	LRLK	AR0,832
	LARP	AR3
	CMPR	0		; AR3 = PCOUNT
	BBZ	HDLP2

*	INITIALISE PHASE ADJUSTMENT ACCUMULATOR

	LARK	AR0,128
	SAR	AR0,ADJ

*	FILL BUF16 WITH NEXT 16 SAMPLES
*	(BUF16+8) SHOULD BE PIXEL 0

	LARK	AR1,BUF16	
PHLP0	CALL	SAMPLE
	CALL	SHFT20
	CALL	SCALE
	LARP	AR1
	SACL	*+
	LARK	AR0,BUF16+16
	CMPR	0
	BBZ	PHLP0		; FILL BUFFER

*	TEST FOR HEADER/IMAGE DATA PATTERN
*	BY WEIGHING THE LAST 7 SAMPLES

	LARK	AR1,BUF16+9
	LARP	AR1
	ZAC
	RPTK	6		; TOTALISE LAST 7 SAMPLES
	ADD	*+
	SBLK	448		; THRESH OF 7*64, SAY
	BGZ	PICAQR		; FIRST IMAGE LINE...

*	PHASE LOCK TO PIXEL NUMBER 0

	LAC	BUF16+8		; PIXEL NO 0
	CALL	PHADJ
	B	HDLP2		; STILL LOCKING TO HEADER

*
* ----	PICTURE AQUISITION ------------------------------
*
*	Initialisation
*

PICAQR	LAC	BUF16+12	; PICK PIXEL NO 4
	LARK	AR4,0		; IMAGE LINE COUNTER
	RXF			; OPEN BUFFER

*	PICLP0 IS THE LINE LOOP
*	LOOP TO THIS POINT WITH ACCL = PIXEL NO 4

PICLP0	CALL	PHADJ		; LOCK TO PIXEL 4

*	RETARD PHASE BY 1/2 A PIXEL

	CALL	SHFT10

*	WAIT FOR PIXEL 40

PICLP1	CALL	SAMPLE
	CALL	SHFT20
	LARK	AR0,40
	LARP	AR3
	CMPR	0		; AR3 = PCOUNT
	BBZ	PICLP1

*	START TRANSFERING

*	RXF			; LINE TX MODE ONLY: START BLOCK
TLOOP1	CALL	SAMPLE		; GET NEXT SAMPLE -> ACCL
	CALL	SHFT20
	CALL	SCALE
	SACL	TEMP		; USE TEMP STORE FOR OUT...
	OUT	TEMP,PA1	; WRITE IT TO BUFFER
	LARP	AR3		; REFERENCE PCOUNT
	LARK	AR0,0		; COMPARE WITH 0
	CMPR	0		; TEST FOR =
	BBZ	TLOOP1		; COMPARISON FAILED, NEXT PIXEL
*	SXF			; LINE TX MODE ONLY: END BLOCK

*	LINE COMPLETE

	LARP	AR4		; PICTURE COMPLETE ?
	MAR	*+
	LRLK	AR0,802
	CMPR	0
	BBNZ	INIT		; COMPLETE

*	GET TO PIXEL 4

	CALL	SAMPLE		; PIX 0
	CALL	SHFT10		; REPAIR PHASE
*	CALL	SHFT20		; PCOUNT = 1
	CALL	SAMPLE		; PIX 1
	CALL	SHFT20		; PCOUNT = 2
	CALL	SAMPLE		; PIX 2
	CALL	SHFT20		; PCOUNT = 3
	CALL	SAMPLE		; PIX 3
	CALL	SHFT20		; PCOUNT = 4
	CALL	SAMPLE		; PIX 4
	CALL	SHFT20
	CALL	SCALE
	B	PICLP0		; NEXT LINE

*
* ---- SUBROUTINES ----------------------------------------
*

*
*	PHADJ - The phase adjustment process
*
*
PHADJ	ADD	ADJ
	ADD	ADJ,1		; + OLD ADJ x 3
	SFR
	SFR
	SACL	ADJ		; FILTER IT IN
	SUBK	128		; -128...127
	SFR
	SFR
	SFR			; DIV 8 -> SAMPLE COUNT
	NEG
	DINT
	SAR	AR7,TEMP
	ADD	TEMP
	SACL	TEMP
	LAR	AR7,TEMP	; ADJUST POINTER
	EINT
	RET


*
*	SCALE adjusts a SAMPLE result to lie in the range 4..250
*
*	      (   (IN - BLKREF)         )
*	OUT = (  --------------- * 246  ) + 4
*	      (       WHTREF            )
*
*	SINCE SUBC WORKS WITH 16 BIT DATA, CANT MPY (IN-BLK)
*	BY 246, THAT WOULD GIVE A 17 BIT RESULT...
*
SCALE	LDPK	0		; BLOCK 2
	SUB	BLKREF
	SACL	TEMP
	LT	TEMP
	MPYK	123		; P := RESULT
	ZAC
	APAC			; ACCL := RESULT
	ADD	WHTREF,2	; + 4 x WHTREF
	BLEZ	SCLLIM		; LOWER LIMIT CLIPPER
	SUB	WHTREF		; IS NUMERATOR > DENOMINATOR ?
	BLEZ	SCLLIM		; ZERO OR 1 RESULT
	ADD	WHTREF		; REPAIR ACCL

*	NOW DIVIDE BY WHTREF TO GET RESULT

	RPTK	15		; 16 CYCLE DIVIDE LOOP
	SUBC	WHTREF		; MULTIPLE CONDITIONAL SUBTRACTS
	SACL	TEMP
	ZALS	TEMP		; CLEAR TOP HALF OF ACCL

*	TEST FOR > 255 & CLIP

	SUBK	255
	BGZ	SCHLIM		; HIGH LIMIT EXCEEDED
	ADDK	255
	RET

SCHLIM	LACK	255
	RET

*	LOWER LIMIT CLIPPER....

SCLLIM	ZAC
	RET			; RETURN ZERO



*
*	SAMPLE waits for buffer to fill beyond threshold of 256+7,
*	runs the two quadrature FIR filters & calculates
*	the signal amplitude -> accl.
*	Call SHIFTn to shuffle the interrupt buffer up.
*	Also returns with updated PCOUNT in AR3.
*	destroys: AR0(=pcount) & DP(=0) & AR3
*

SAMPLE	LARP	AR7		; LOOK AT SAMPLE BUFFER POINTER
	LRLK	AR0,BLOCK1+7	; GOT ENOUGH YET ?
SMPLP0	CMPR	2		; IS AR7 > AR0 ? TC=1 IF SO
	BBZ	SMPLP0		; WAIT TILL BUFFER IS FULL
	LDPK	4		; PAGE 4 = BOTTOM OF BLOCK 0

*	FILTER 1	

	MPYK	0		; P:= 0
	LARP	AR0		; USE AR0
	LRLK	AR0,BLOCK0+0	; POINT TO BUFFER
	ZAC
	RPTK	63
	MAC	COEFS+0,*+
	RPTK	63
	MAC	COEFS+64,*+
	RPTK	63
	MAC	COEFS+128,*+
	RPTK	62
	MAC	COEFS+192,*+
	APAC
	SACH	0		; SAVE RESULT/65536 @ (0)

*	FILTER 2

	MPYK	0		; P:= 0
	LARP	AR0		; USE AR0
	LRLK	AR0,BLOCK0+7	; POINT TO BUFFER
	ZAC
	RPTK	63
	MAC	COEFS+0,*+
	RPTK	63
	MAC	COEFS+64,*+
	RPTK	63
	MAC	COEFS+128,*+
	RPTK	62
	MAC	COEFS+192,*+
	APAC
	SACH	1		; SAVE RESULT/65536 @ (1)

*	CALC SUM OF SQUARES

	MPYK	0
	ZAC			; ACCL = P = 0
	SQRA	0		; P := (0)x(0)
	SQRA	1		; ACCL := S(0)xS(0) ; P := S(7)xS(7)
	APAC			; ACCL := S(0)xS(0) + S(7)xS(7)

*	COMPENSATE FOR FILTER GAIN.
*	FILTER GAIN = 4, SO... DIV 16

	RPTK	3		; REPEAT 4 TIMES
	SFR			; SHIFT RIGHT

*	CALC SQUARE ROOT OF 18 BIT +VE NUMBER

	MPYK	0		; P := 0 [* x (0)]
	LARP	AR0		; USE AR0

	LRLK	AR0,512		; FIRST GUESS = 512
	SAR	AR0,1		; SAVE @ GUESS
	SQRA	1		; P := GUESS SQUARED
	SPAC			; ACCL := ACCL - P
	BC	ROOT8		; NOT TOO GREAT
	LARK	AR0,0		; RESET BIT 9

ROOT8	ADRK	128
	ADRK	128		; ADRK 256
	SAR	AR0,1		; SAVE @ GUESS
	SQRA	1		; P := GUESS SQUARED
	SPAC			; ACCL := ACCL - P
	BC	ROOT7		; NOT TOO GREAT
	SBRK	128
	SBRK	128

ROOT7	ADRK	128
	SAR	AR0,1		; SAVE @ GUESS
	SQRA	1		; P := GUESS SQUARED
	SPAC			; ACCL := ACCL - P
	BC	ROOT6		; NOT TOO GREAT
	SBRK	128

ROOT6	ADRK	64
	SAR	AR0,1		; SAVE @ GUESS
	SQRA	1		; P := GUESS SQUARED
	SPAC			; ACCL := ACCL - P
	BC	ROOT5		; NOT TOO GREAT
	SBRK	64

ROOT5	ADRK	32
	SAR	AR0,1		; SAVE @ GUESS
	SQRA	1		; P := GUESS SQUARED
	SPAC			; ACCL := ACCL - P
	BC	ROOT4		; NOT TOO GREAT
	SBRK	32

ROOT4	ADRK	16
	SAR	AR0,1		; SAVE @ GUESS
	SQRA	1		; P := GUESS SQUARED
	SPAC			; ACCL := ACCL - P
	BC	ROOT3		; NOT TOO GREAT
	SBRK	16

ROOT3	ADRK	8
	SAR	AR0,1		; SAVE @ GUESS
	SQRA	1		; P := GUESS SQUARED
	SPAC			; ACCL := ACCL - P
	BC	ROOT2		; NOT TOO GREAT
	SBRK	8 

ROOT2	ADRK	4
	SAR	AR0,1		; SAVE @ GUESS
	SQRA	1		; P := GUESS SQUARED
	SPAC			; ACCL := ACCL - P
	BC	ROOT1		; NOT TOO GREAT
	SBRK	4 

ROOT1	ADRK	2
	SAR	AR0,1		; SAVE @ GUESS
	SQRA	1		; P := GUESS SQUARED
	SPAC			; ACCL := ACCL - P
	BC	ROOT0		; NOT TOO GREAT
	SBRK	2 

ROOT0	ADRK	1
	SAR	AR0,1		; SAVE @ GUESS
	SQRA	1		; P := GUESS SQUARED
	SPAC			; ACCL := ACCL - P
	BC	ROOTQ		; NOT TOO GREAT
	SBRK	1 

ROOTQ	LAC	1		; 9 bit result in accl

*	AND FINALLY, INCREMENT THE PIXEL COUNTER
*	MODULUS 840

	LDPK	0
	LAR	AR3,PCOUNT
	LARP	AR3
	MAR	*+
	SAR	AR3,PCOUNT
	LRLK	AR0,840
	CMPR	0
	BBZ	SMPLQ
	LARK	AR3,0
	SAR	AR3,PCOUNT
SMPLQ	RET


*
*	SHFT20 - 1pixel shift
*	Shuffle the interrupt buffer allong 20 clicks
*	Destroys AR0
*	Do first 256+7-20 shifts
*
SHFT20	LARP	AR0		; USE AR0
	LRLK	AR0,BLOCK0	; DESTINATION
	RPTK	60		; 61 TIMES
	BLKD	BLOCK0+20+0,*+	; SOURCE = DEST+20
	RPTK	60		; 61 TIMES
	BLKD	BLOCK0+20+61,*+	; SOURCE = DEST+20
	RPTK	60		; 61 TIMES
	BLKD	BLOCK0+20+122,*+	; SOURCE = DEST+20
	RPTK	59		; 60 TIMES
	BLKD	BLOCK0+20+183,*+	; SOURCE = DEST+20

*	Now to re-align the interrupt buffer

	IDLE
	DINT
	RPTK	63		; 64 TIMES
	BLKD	BLOCK1+7,*+	; SOURCE = DEST+20
	LARP	AR7		; RESET AR7
	SBRK	20		; SHIFT IT TOO
	EINT

	RET


*
*	SHFT10 - 1/2 pixel shift
*	Shuffle the interrupt buffer allong 10 clicks
*	Destroys AR0
*	Do first 256+7-10 shifts
*
SHFT10	LARP	AR0		 ; USE AR0
	LRLK	AR0,BLOCK0	 ; DESTINATION
	RPTK	60		 ; 61 TIMES
	BLKD	BLOCK0+10+0,*+	 ; SOURCE = DEST+10
	RPTK	60		 ; 61 TIMES
	BLKD	BLOCK0+10+61,*+	 ; SOURCE = DEST+10
	RPTK	60		 ; 61 TIMES
	BLKD	BLOCK0+10+122,*+ ; SOURCE = DEST+10
	RPTK	69		 ; 70 TIMES
	BLKD	BLOCK0+10+183,*+ ; SOURCE = DEST+10

*	Now to re-align the interrupt buffer

	IDLE
	DINT
	RPTK	63		; 64 TIMES
	BLKD	BLOCK1+7,*+	; SOURCE = DEST+20
	LARP	AR7		; RESET AR7
	SBRK	10		; SHIFT IT TOO
	EINT

	RET


*
*	SHFT56 - 2.8pixel shift
*	Shuffle the interrupt buffer allong 56 clicks
*	Destroys AR0
*	Do first 256+7-56 shifts
*
SHFT56	LARP	AR0		; USE AR0
	LRLK	AR0,BLOCK0	; DESTINATION
	RPTK	68		; 69 TIMES
	BLKD	BLOCK0+56,*+	; SOURCE = DEST+56
	RPTK	68		; 69 TIMES
	BLKD	BLOCK0+56+69,*+	; SOURCE = DEST+56
	RPTK	68		; 69 TIMES
	BLKD	BLOCK0+56+138,*+	; SOURCE = DEST+56
	IDLE
	DINT
	RPTK	63		; 64 TIMES
	BLKD	BLOCK1+7,*+	; SOURCE = DEST+56
	LARP	AR7		; RESET AR7
	SBRK	56		; SHIFT IT TOO
	EINT
	RET


*
* ----	FIR FILTER COEFICIENT LIST  ---------------------
*

COEFS	DATA	-1497	; 1
	DATA	1264	; 2
	DATA	898	; 3
	DATA	185	; 4
	DATA	-211	; 5
	DATA	-421	; 6
	DATA	-423	; 7
	DATA	-321	; 8
	DATA	-139	; 9
	DATA	52	; 10
	DATA	237	; 11
	DATA	376	; 12
	DATA	466	; 13
	DATA	492	; 14
	DATA	467	; 15
	DATA	392	; 16
	DATA	289	; 17
	DATA	170	; 18
	DATA	58	; 19
	DATA	-31	; 20
	DATA	-79	; 21
	DATA	-81	; 22
	DATA	-29	; 23
	DATA	69	; 24
	DATA	204	; 25
	DATA	356	; 26
	DATA	507	; 27
	DATA	631	; 28
	DATA	714	; 29
	DATA	739	; 30
	DATA	704	; 31
	DATA	612	; 32
	DATA	479	; 33
	DATA	324	; 34
	DATA	174	; 35
	DATA	53	; 36
	DATA	-14	; 37
	DATA	-12	; 38
	DATA	63	; 39
	DATA	204	; 40
	DATA	394	; 41
	DATA	603	; 42
	DATA	800	; 43
	DATA	953	; 44
	DATA	1035	; 45
	DATA	1028	; 46
	DATA	928	; 47
	DATA	745	; 48
	DATA	504	; 49
	DATA	240	; 50
	DATA	-7	; 51
	DATA	-199	; 52
	DATA	-303	; 53
	DATA	-301	; 54
	DATA	-190	; 55
	DATA	16	; 56
	DATA	283	; 57
	DATA	567	; 58
	DATA	819	; 59
	DATA	991	; 60
	DATA	1047	; 61
	DATA	965	; 62
	DATA	744	; 63
	DATA	406	; 64
	DATA	-5	; 65
	DATA	-435	; 66
	DATA	-825	; 67
	DATA	-1118	; 68
	DATA	-1266	; 69
	DATA	-1248	; 70
	DATA	-1069	; 71
	DATA	-758	; 72
	DATA	-365	; 73
	DATA	36	; 74
	DATA	371	; 75
	DATA	574	; 76
	DATA	588	; 77
	DATA	391	; 78
	DATA	-9	; 79
	DATA	-575	; 80
	DATA	-1234	; 81
	DATA	-1902	; 82
	DATA	-2482	; 83
	DATA	-2892	; 84
	DATA	-3066	; 85
	DATA	-2978	; 86
	DATA	-2638	; 87
	DATA	-2102	; 88
	DATA	-1457	; 89
	DATA	-815	; 90
	DATA	-295	; 91
	DATA	-5	; 92
	DATA	-22	; 93
	DATA	-379	; 94
	DATA	-1059	; 95
	DATA	-1989	; 96
	DATA	-3054	; 97
	DATA	-4106	; 98
	DATA	-4990	; 99
	DATA	-5563	; 100
	DATA	-5722	; 101
	DATA	-5421	; 102
	DATA	-4681	; 103
	DATA	-3598	; 104
	DATA	-2326	; 105
	DATA	-1066	; 106
	DATA	-36	; 107
	DATA	561	; 108
	DATA	569	; 109
	DATA	-92	; 110
	DATA	-1409	; 111
	DATA	-3264	; 112
	DATA	-5444	; 113
	DATA	-7655	; 114
	DATA	-9560	; 115
	DATA	-10813	; 116
	DATA	-11109	; 117
	DATA	-10222	; 118
	DATA	-8040	; 119
	DATA	-4591	; 120
	DATA	-46	; 121
	DATA	5289	; 122
	DATA	11001	; 123
	DATA	16610	; 124
	DATA	21623	; 125
	DATA	25582	; 126
	DATA	28117	; 127
	DATA	28990	; 128
	DATA	28117	; 129
	DATA	25582	; 130
	DATA	21623	; 131
	DATA	16610	; 132
	DATA	11001	; 133
	DATA	5289	; 134
	DATA	-46	; 135
	DATA	-4591	; 136
	DATA	-8040	; 137
	DATA	-10222	; 138
	DATA	-11109	; 139
	DATA	-10813	; 140
	DATA	-9560	; 141
	DATA	-7655	; 142
	DATA	-5444	; 143
	DATA	-3264	; 144
	DATA	-1409	; 145
	DATA	-92	; 146
	DATA	569	; 147
	DATA	561	; 148
	DATA	-36	; 149
	DATA	-1066	; 150
	DATA	-2326	; 151
	DATA	-3598	; 152
	DATA	-4681	; 153
	DATA	-5421	; 154
	DATA	-5722	; 155
	DATA	-5563	; 156
	DATA	-4990	; 157
	DATA	-4106	; 158
	DATA	-3054	; 159
	DATA	-1989	; 160
	DATA	-1059	; 161
	DATA	-379	; 162
	DATA	-22	; 163
	DATA	-5	; 164
	DATA	-295	; 165
	DATA	-815	; 166
	DATA	-1457	; 167
	DATA	-2102	; 168
	DATA	-2638	; 169
	DATA	-2978	; 170
	DATA	-3066	; 171
	DATA	-2892	; 172
	DATA	-2482	; 173
	DATA	-1902	; 174
	DATA	-1234	; 175
	DATA	-575	; 176
	DATA	-9	; 177
	DATA	391	; 178
	DATA	588	; 179
	DATA	574	; 180
	DATA	371	; 181
	DATA	36	; 182
	DATA	-365	; 183
	DATA	-758	; 184
	DATA	-1069	; 185
	DATA	-1248	; 186
	DATA	-1266	; 187
	DATA	-1118	; 188
	DATA	-825	; 189
	DATA	-435	; 190
	DATA	-5	; 191
	DATA	406	; 192
	DATA	744	; 193
	DATA	965	; 194
	DATA	1047	; 195
	DATA	991	; 196
	DATA	819	; 197
	DATA	567	; 198
	DATA	283	; 199
	DATA	16	; 200
	DATA	-190	; 201
	DATA	-301	; 202
	DATA	-303	; 203
	DATA	-199	; 204
	DATA	-7	; 205
	DATA	240	; 206
	DATA	504	; 207
	DATA	745	; 208
	DATA	928	; 209
	DATA	1028	; 210
	DATA	1035	; 211
	DATA	953	; 212
	DATA	800	; 213
	DATA	603	; 214
	DATA	394	; 215
	DATA	204	; 216
	DATA	63	; 217
	DATA	-12	; 218
	DATA	-14	; 219
	DATA	53	; 220
	DATA	174	; 221
	DATA	324	; 222
	DATA	479	; 223
	DATA	612	; 224
	DATA	704	; 225
	DATA	739	; 226
	DATA	714	; 227
	DATA	631	; 228
	DATA	507	; 229
	DATA	356	; 230
	DATA	204	; 231
	DATA	69	; 232
	DATA	-29	; 233
	DATA	-81	; 234
	DATA	-79	; 235
	DATA	-31	; 236
	DATA	58	; 237
	DATA	170	; 238
	DATA	289	; 239
	DATA	392	; 240
	DATA	467	; 241
	DATA	492	; 242
	DATA	466	; 243
	DATA	376	; 244
	DATA	237	; 245
	DATA	52	; 246
	DATA	-139	; 247
	DATA	-321	; 248
	DATA	-423	; 249
	DATA	-421	; 250
	DATA	-211	; 251
	DATA	185	; 252
	DATA	898	; 253
	DATA	1264	; 254
	DATA	-1497	; 255



*
* ---- 300Hz XC COEFs ----------------
*

XC300	DATA	 0
	DATA	-1
	DATA	 0
	DATA	 1
	DATA	 0
	DATA	-1
	DATA	 0
	DATA	 1
	DATA	 0
	DATA	-1
	DATA	 0
	DATA	 1
	DATA	 0
	DATA	-1
	DATA	 0
	DATA	 1


*
* ----	CROSS CORRELATION REFERENCE TABLE  ---------------------
*	WITH OLDEST DATA IN LO MEMORY
*

XCTAB	DATA	 0
	DATA	-1
	DATA	-1	; 35-128
	DATA	 0	; 126-128
	DATA	 1	; 217-128
	DATA	 1	; 216-128
	DATA	 0	; 124-128
	DATA	-1	; 35-128
	DATA	-1	; 53-128
	DATA	 0	; 152-128
	DATA	 1	; 218-128
	DATA	 1	; 190-128
	DATA	 0	; 93-128
	DATA	-1	; 25-128
	DATA	-1	; 74-128
	DATA	 1	; 220-128
	DATA	 1	; 164-128
	DATA	 0	; 181-128
	DATA	-1	; 67-128
	DATA	-1	; 29-128
	DATA	 0	; 104-128
	DATA	 1	; 202-128
	DATA	 1	; 212-128
	DATA	 0	; 134-128
	DATA	-1	; 44-128
	DATA	-1	; 38-128
	DATA	 0	; 136-128
	DATA	 1	; 226-128
	DATA	 1	; 244-128
	DATA	 1	; 244-128
	DATA	 1	; 246-128
	DATA	 1	; 250-128

	END
